# -*- coding: utf-8 -*-

from paclair.api.abstract_clair_requests import AbstractClairRequests
from paclair.struct import InsensitiveCaseDict


class ClairRequestsV1(AbstractClairRequests):
    """
    Request Clair helper
    """

    _CLAIR_ANALYZE_URI = "/v1/layers/{}?features&vulnerabilities"
    _CLAIR_POST_URI = "/v1/layers"
    _CLAIR_DELETE_URI = "/v1/layers/{}"

    def get_ancestry_json(self, ancestry):
        """
        Analyse an ancestry

        :param ancestry: ancestry (name) to analyse
        :return: json
        """
        return self.get_layer(ancestry)

    def post_ancestry(self, ancestry):
        """
        Post ancestry to Clair

        :param ancestry: ancestry to push
        """
        additional_data = {}
        for layer in ancestry.layers:
            if layer.headers is not None:
                additional_data['Headers'] = layer.headers
            data = self.to_clair_post_data(layer.name, layer.path, ancestry.clair_format, **additional_data)
            self.post_layer(data)
            additional_data = {'ParentName': layer.name}

    def delete_ancestry(self, ancestry):
        """
        Delete ancestry from Clair

        :param ancestry: ancestry to delete
        """
        for layer in ancestry.layers[::-1]:
            self.delete_layer(layer.name)

    def get_layer(self, name):
        """
        Analyse a layer

        :param name: layer's name
        :return: json
        """
        response = self._request('GET', self._CLAIR_ANALYZE_URI.format(name))
        return response.json()

    def post_layer(self, data):
        """
        POST layer to Clair

        :param data: data to send to clair
        """
        self.logger.debug("Sending to Clair: {}".format(data))
        return self._request('POST', self._CLAIR_POST_URI, json=data)

    def delete_layer(self, name):
        """
        Delete layer

        :param name: layer's name
        """
        return self._request('DELETE', self._CLAIR_DELETE_URI.format(name))

    @staticmethod
    def to_clair_post_data(name, path, clair_format, **kwargs):
        """
        Helper

        :param name: resource's name
        :param path: resource path
        :param clair_format: Clair format
        :params kwargs: additional params
        :return: json to post to Clair
        """
        data = {"Layer": {"Name": name, "Path": path, "Format": clair_format}}
        data["Layer"].update(kwargs)
        return data

    def _iter_features(self, clair_json):
        for feature in clair_json.get("Layer", {}).get("Features", {}):
            yield InsensitiveCaseDict(feature)



import unittest
from unittest.mock import patch

import requests_mock

from paclair.api.clair_requests_v1 import ClairRequestsV1
from paclair.docker.docker_image import DockerImage
from paclair.exceptions import ResourceNotFoundException
from paclair.plugins.docker_plugin import DockerPlugin


class TestDockerPlugin(unittest.TestCase):
    """docstring for TestDockerPlugin"""

    def setUp(self):
        self.docker = DockerPlugin(ClairRequestsV1('http://localhost:6060'))

    @requests_mock.mock()
    def test_push(self, m):
        get_layers = ['fffffffffffffffffffffffffffffffffffffffff']
        m.post(self.docker.clair.url + self.docker.clair._CLAIR_POST_URI,
               status_code=201, reason="OK")
        with patch.object(DockerImage, "get_layers", return_value=get_layers):
            with patch.object(DockerImage, "token", return_value="MYTOKEN"):
                self.docker.push('monimage:tags')

    @requests_mock.mock()
    def test_push_404(self, m):
        get_layers = ['fffffffffffffffffffffffffffffffffffffffff']
        m.post(self.docker.clair.url + self.docker.clair._CLAIR_POST_URI,
               status_code=404, reason="Not Found")
        with patch.object(DockerImage, "get_layers", return_value=get_layers):
            with patch.object(DockerImage, "token", return_value="MYTOKEN"), self.assertRaises(ResourceNotFoundException):
                self.docker.push('monimage:tags')


# -*- coding: utf-8 -*-
import base64
import datetime

from elasticsearch import Elasticsearch, RequestsHttpConnection

from paclair.ancestries.generic import GenericAncestry, Layer
from paclair.plugins.abstract_plugin import AbstractPlugin
from paclair.exceptions import ResourceNotFoundException


class EsPlugin(AbstractPlugin):
    """
    Elasticsearch plugin
    """

    __SOURCE_URL = "{}/{}/{}/{}/_source?_source_include=sysinfo_clair"

    def __init__(self, clair, hosts, index, doc_type, suffix=None, timedelta=None):
        """
        Constructor

        :param clair: ClairRequest object
        :param hosts: elasticsearch hosts ex:[{'host': '172.18.8.10', 'port': 9200}]
        :param index: elasticsearch index
        :param doc_type: elasticsearch doc_type
        :param suffix: index suffix (ex: one index a day)
        :param timedelta: timedelta from today for suffix
        """
        super().__init__(clair, "Legacy")
        self._es = Elasticsearch(hosts, connection_class=RequestsHttpConnection)
        self.index = index
        self.doc_type = doc_type
        if suffix is not None:
            timedelta = timedelta or {}
            self.index += (datetime.datetime.today() + datetime.timedelta(**timedelta)).strftime(suffix)

    def push(self, name):
        ancestry = self.create_ancestry(name)
        self.logger.debug("Remove ancestry {} from Clair's Database.".format(name))
        try:
            self.clair.delete_ancestry(ancestry)
        except ResourceNotFoundException:
            self.logger.debug("Ancestry {} not yet in Clair's Database.".format(name))

        return self.clair.post_ancestry(ancestry)

    def create_ancestry(self, name):
        # get ID
        search = {"size": 1, "sort": {'@timestamp': "desc"}, "_source": False,
                  "query": {'match_phrase': {'hostname': name}}}
        result = self._es.search(index=self.index, doc_type=self.doc_type, body=search)['hits']
        # no result
        if result['total'] == 0:
            raise ResourceNotFoundException("{} not found".format(name))

        id_name = result['hits'][0]['_id']
        path = self.__SOURCE_URL.format(self._es.transport.get_connection().host, self.index, self.doc_type, id_name)

        # Authentication
        auth = self._es.transport.get_connection().session.auth
        headers = None
        if auth is not None:
            digest = "{}:{}".format(*auth)
            digest = base64.b64encode(digest.encode("utf-8"))
            headers = {'Authorization': 'Basic {}'.format(digest.decode('utf-8'))}
        return GenericAncestry(name, self.clair_format, [Layer(name, name, path, headers)])



# -*- coding: utf-8 -*-
from paclair.api.clair_requests_v1 import ClairRequestsV1
from paclair.api.clair_requests_v3 import ClairRequestsV3
from paclair.logged_object import LoggedObject
from paclair.exceptions import ConfigurationError
import importlib
import yaml


class ConfigReader(LoggedObject):
    """
    Read configuration file
    """

    def __init__(self, filename):
        """
        Constructor

        :param filename: configuration filename
        """
        super(ConfigReader, self).__init__()
        self.filename = filename

    def read_section(self, section):
        """
        Read section's configuration

        :param section: section's name
        """
        with open(self.filename, 'r') as f:
            # Load
            self.logger.debug("Reading section %s in file %s", section, self.filename)
            conf = yaml.safe_load(f)

            # Check structure
            if section not in conf:
                self.logger.error("No section %s in configuration file", section)
                return {}
            return conf[section]

    def read_plugins(self, plugin_section="Plugins"):
        """
        Read plugins

        :param plugin_section: plugins' section's name
        """
        plugins = self.read_section(plugin_section)
        clair_conf = self.read_section("General")
        if not clair_conf:
            raise ConfigurationError("Can't read Clair's configuration")

        # Read clair api class
        clair_api_version = clair_conf.pop("clair_api_version", 1)
        if clair_api_version == 3:
            clair = ClairRequestsV3(**clair_conf)
        else:
            clair = ClairRequestsV1(**clair_conf)

        result = {}
        for plugin, conf in plugins.items():
            try:
                self.logger.debug('Reading plugin %s', plugin)
                self.logger.debug('Configuration %s', conf)

                plugin_class = self._get_class(conf.pop('class'))
                result[plugin] = plugin_class(clair, **conf)
            except (ValueError, KeyError):
                self.logger.error("Can't read plugin %s", plugin)

        return result

    @staticmethod
    def _get_class(kls):
        """
        Instantiate a class from a String

        :param kls: class name
        """
        mod, _, cls = kls.rpartition('.')

        # Import module
        mod = importlib.import_module(mod)

        return getattr(mod, cls)



#-*- coding: UTF-8 -*-
from collections import MutableMapping


class InsensitiveCaseDict(MutableMapping):
    """
    An insensitive case dict
    """

    def __init__(self, data=None, **kwargs):
        """
        Init the dictionnary

        :param data: an existing dictionnary
        :param kwargs: other arguments
        """
        self._storage = {}
        data = data or {}
        self.update(data, **kwargs)

    def __getitem__(self, item):
        return self._storage[self._lower(item)]

    def __setitem__(self, key, value):
        self._storage[self._lower(key)] = value

    def __delitem__(self, key):
        del self._storage[self._lower(key)]

    def __iter__(self):
        return iter(self._storage)

    def __len__(self):
        return len(self._storage)

    @staticmethod
    def _lower(item):
        """
        Lowercase item

        :param item: item to lowercase
        :return: Lowercased item
        """
        if isinstance(item, str):
            return item.lower()
        return item



# -*- coding: utf-8 -*-

import re

from paclair import DOCKER_HUB_DOMAIN
from paclair import REGEX
from paclair.ancestries.docker import DockerAncestry
from paclair.docker.docker_image import DockerImage
from paclair.docker.docker_registry import DockerRegistry
from paclair.exceptions import ResourceNotFoundException
from paclair.plugins.abstract_plugin import AbstractPlugin


class DockerPlugin(AbstractPlugin):
    """
    Docker plugin
    """

    def __init__(self, clair, registries=None):
        """
        Constructor

        :param clair: ClairRequests object
        :param registries: registries' configuration {'registry1_domain': {'conf': ...},
                                                      'registry2_domain': {'conf': ...}}
        """
        super().__init__(clair, "Docker")
        registries = registries or {}
        self.__registries = {domain: DockerRegistry(domain, **conf) for domain, conf in registries.items()}
        self.__docker_hub = DockerRegistry(DOCKER_HUB_DOMAIN)
        self._pattern = re.compile(REGEX['domain'] + REGEX['name'] + REGEX['tag'])
        self._domain_pattern = re.compile(r'(?P<repository>[a-zA-Z0-9-]*)\.(?P<domain>[a-zA-Z0-9-.]*)$')

    def create_docker_image(self, name):
        """
        Create docker image

        :param name: image's name
        :return: paclair.docker.DockerImage
        """
        matcher = self._pattern.match(name)
        if not matcher:
            raise ResourceNotFoundException("Incorrect image name: {}".format(name))

        # Base docker image
        if matcher.group("domain") is None:
            return DockerImage("library/" + matcher.group("name"), self.__docker_hub,
                               tag=matcher.group("tag") or 'latest')

        domain_regex_matcher = self._domain_pattern.search(matcher.group("domain") or "")
        # Repo docker
        if domain_regex_matcher is None:
            return DockerImage("{}/{}".format(matcher.group("domain"), matcher.group("name")), self.__docker_hub,
                               tag=matcher.group("tag") or 'latest')

        # Find the registry
        repo = ""
        if domain_regex_matcher.group("domain") in self.__registries:
            registry = self.__registries[domain_regex_matcher.group("domain")]
            repo = domain_regex_matcher.group("repository") or ""
        elif matcher.group("domain") in self.__registries:
            registry = self.__registries[matcher.group("domain")]
        else:
            registry = DockerRegistry(matcher.group("domain"))
        return DockerImage(matcher.group("name"), registry, repo, tag=matcher.group("tag") or 'latest')

    def create_ancestry(self, name):
        return DockerAncestry(self.create_docker_image(name))

    def analyse(self, name, output=None):
        ancestry = self.create_ancestry(name)
        return super().analyse(ancestry.name, output)



import logging
import unittest

import requests_mock

from paclair.api.clair_requests_v1 import ClairRequestsV1
from paclair.exceptions import ResourceNotFoundException
from paclair.plugins.http_plugin import HttpPlugin


class TestHttpPlugin(unittest.TestCase):
    """
    Test de l'objet CfPlugin
    """
    
    artifacURI = 'http://artifac'
    artifacVERIFY = 'verifymock'
    clairURI = 'http://clair'

    def setUp(self):
        self.cf = HttpPlugin(ClairRequestsV1(self.clairURI), "cflinuxfs", self.artifacURI, self.artifacVERIFY)


    @requests_mock.mock()
    def test_push_not_found(self, m):
        """
        Test de la méthode push quand l'image n'existe pas
        """
        # mock artifactory reponse
        m.head(self.artifacURI + "/tutu", status_code = 404)
        with self.assertRaises(ResourceNotFoundException):
            self.cf.push("tutu")

    @requests_mock.mock()
    def test_push_found(self, m):
        """
        Test de la méthode push quand l'image existe
        """
        # mock artifactory reponse
        m.head(self.artifacURI + "/tutu", status_code = 200)
        # mock clair reponse
        m.post(self.clairURI + '/v1/layers')
        self.cf.push("tutu")
        clair_data = {'Layer': {'Path': self.artifacURI + "/tutu", 'Format': 'cflinuxfs', 'Name': 'tutu'}}
        self.assertEqual(m.last_request.json(), clair_data)

if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



# -*- coding: utf-8 -*-
from paclair.logged_object import LoggedObject


class GenericAncestry(LoggedObject):
    """
    Ancestry object
    """

    def __init__(self, name, clair_format, layers=None):
        """
        Constructor

        :param name: ancestry's name
        :param clair_format: clair format
        :param layers: [{"name": str, "hash": str, "path": str, "headers":{}}]
        """
        super().__init__()
        self.logger.debug("Creating {} ancestry".format(name))
        self.name = name
        self.clair_format = clair_format
        self.layers = layers or []

    def to_json(self):
        """

        :return: dict
        """
        return {"ancestry_name": self.name, "format": self.clair_format,
                "layers": [{'hash': l.lhash, 'path': l.path, 'headers': l.headers}for l in self.layers]}


class Layer(LoggedObject):
    """
    Layer object
    """

    def __init__(self, name, lhash, path, headers=None, parent=""):
        """
        Constructor

        :param name: layer name
        :param lhash: hash
        :param path: path
        :param headers: headers
        :param parent: parent name
        """
        super().__init__()
        self.name = name
        self.lhash = lhash
        self.path = path
        self.headers = headers
        self.parent = parent



# -*- coding: utf-8 -*-

"""
Docker_image module
"""

import hashlib
from paclair.logged_object import LoggedObject


class DockerImage(LoggedObject):
    """
    A Docker Image
    """

    def __init__(self, name, registry, repository="", tag='latest'):
        """
        Constructor

        :param name: docker image's name (ex: ubuntu, centos, ...)
        :param repository: repository's name
        :param tag: image's tag
        :param registry: Docker registry
        """
        super(DockerImage, self).__init__()
        self.name = name
        self.logger.debug("INITCLASS:NAMEIMAGE:{name}".format(name=self.name))
        self.tag = tag
        self.logger.debug("INITCLASS:TAG:{tag}".format(tag=self.tag))
        self.registry = registry
        self.repository = repository
        self._manifest = None
        self._sha = None
        self.logger.debug("INITCLASS:REPOSITORY:{repository}".format(repository=self.repository))

    @property
    def token(self):
        """
        Token for this image

        :return:
        """
        return self.registry.get_token(self)

    @property
    def sha(self):
        """
        Sha256 of the layers list (used for clair layer_name)

        :return: sha256
        """
        if self._sha is None:
            m = hashlib.sha256(''.join(self.get_layers()).encode('utf-8'))
            self._sha = m.hexdigest()
        return self._sha

    @property
    def short_sha(self):
        """
        Sha short version (12 characters)
        """
        return self.sha[:12]

    @property
    def manifest(self):
        """
        Get manifest

        :returns dict:
        """
        if self._manifest is None:
            self._manifest = self.registry.get_manifest(self)
        return self._manifest

    def get_layers(self):
        """
        Get ordered layers

        :returns list:
        """
        manifest = self.manifest
        layers = []
        # Check Version
        if manifest["schemaVersion"] == 2:
            fs_layers = manifest['layers']
            digest_field = 'digest'
        else:
            fs_layers = manifest.get('fsLayers', [])[::-1]
            digest_field = 'blobSum'

        # List layers
        for fs_layer in fs_layers:
            if fs_layer[digest_field] not in layers:
                layers.append(fs_layer[digest_field])
        return layers



# -*- coding: utf-8 -*-

"""
Docker_registry module
"""

import requests
import re
from paclair.logged_object import LoggedObject
from paclair.exceptions import RegistryAccessError


class DockerRegistry(LoggedObject):
    """
    A Docker Registry
    """
    BASE_API_URL = '{registry.protocol}://{registry.domain}{registry.api_prefix}'
    MANIFEST_URI = '/v2/{image.name}/manifests/{image.tag}'
    BLOBS_URI = '/v2/{image.name}/blobs/{digest}'
    TOKEN_REGEX = "Bearer realm=\"(?P<realm>.*)\",service=\"(?P<service>.*)\""

    def __init__(self, domain, token_url=None, api_prefix="", protocol="https", auth=None, verify=True):
        """
        Constructor

        :param api_prefix: api prefix
        :param domain: domain registry (ex : registry.hub.docker.com)
        :param token_url: token url (ex : https://auth.docker.io)
        :param auth: (user, password)
        :param verify: requests ssl verify
        """
        super().__init__()
        self.domain = domain
        self.logger.debug("INITCLASS:DOMAIN:{domain}".format(domain=self.domain))
        self.api_prefix = api_prefix
        self.logger.debug("INITCLASS:API_PREFIX:{}".format(self.api_prefix))
        self.protocol = protocol
        self.logger.debug("INITCLASS:API_PROTOCOL:{}".format(self.protocol))
        self.auth = tuple(auth or [])
        self.verify = verify
        self.logger.debug("INITCLASS:API_VERIFY:{}".format(self.verify))
        self.__token_url = token_url
        self.logger.debug("INITCLASS:TOKEN_URL:{}".format(self.__token_url))

    @property
    def token_url(self):
        """
        Find token url if not defined
        """
        if self.__token_url is None:
            url = self.BASE_API_URL.format(registry=self) + "/v2/"
            self.logger.debug("REQUEST_BASE_API_URL_FOR_TOKEN_ENDPOINT:URL:{}".format(url))

            response = requests.get(url, verify=self.verify)
            if not "www-authenticate" in response.headers:
                self.logger.error("REQUEST_TOKEN:HTTPCODEERROR:{}".format(response.status_code))
                raise RegistryAccessError("Error access to : {} \nCode Error : {}".format(url, response.status_code))

            # Find realm and service
            matcher = re.match(self.TOKEN_REGEX, response.headers["www-authenticate"])
            if matcher is None:
                raise RegistryAccessError("Can't find token url")
            self.__token_url = "{}?client_id=paclair&service={}&scope=repository:{}:pull".format(
                matcher.group("realm"), matcher.group("service"), "{image.name}")
            self.logger.debug("TOKEN_URL:{}".format(self.__token_url))

        return self.__token_url

    def get_base_api_url(self, docker_image):
        """
        Get base api url

        :param docker_image: paclair.docker.DockerImage
        :return:
        """
        # Format for registry
        url = self.BASE_API_URL.format(registry=self)
        # Format if image's parameters are required ex: prefix_api=/api/{image.repo}
        return url.format(image=docker_image)

    def get_manifest_url(self, docker_image):
        """
        Manifest's url

        :param docker_image: paclair.docker.DockerImage
        :return: manifest's url
        """
        url = self.get_base_api_url(docker_image) + self.MANIFEST_URI.format(image=docker_image)
        return url

    def get_blobs_url(self, docker_image, digest):
        """
        Blobs's url

        :param docker_image: paclair.docker.DockerImage
        :param digest: digest
        :return: blobs's url
        """
        url = self.get_base_api_url(docker_image) + self.BLOBS_URI.format(image=docker_image, digest=digest)
        return url

    def get_token(self, docker_image):
        """
        Get token

        :param docker_image: paclair.docker.DockerImage
        :returns str:
        """
        # Define url
        url = self.token_url.format(registry=self, image=docker_image)
        self.logger.debug("REQUEST_TOKEN:URL:{url}".format(url=url))

        # Get token
        resp = requests.get(url, verify=self.verify, auth=self.auth)
        if not resp.ok:
            self.logger.error("REQUEST_TOKEN:HTTPCODEERROR:{}".format(resp.status_code))
            raise RegistryAccessError("Error access to : {url} \nCode Error : {status_code}".format(
                url=url, status_code=resp.status_code))

        token = resp.json()['token']
        self.logger.debug("TOKEN:{token}".format(token=token))
        return token

    def get_manifest(self, docker_image):
        """
        Get manifest

        :param docker_image: paclair.docker.DockerImage
        :return: manifest
        """
        # Define url
        url = self.get_manifest_url(docker_image)
        self.logger.debug("REQUESTMANIFESTS:{url}".format(url=url))

        # Get token
        token = self.get_token(docker_image)
        resp = requests.get(url, verify=self.verify, headers={"Authorization": "Bearer {}".format(token)})

        if not resp.ok:
            self.logger.error("MANIFESTS:HTTPCODEERROR:{}".format(resp.status_code))
            raise RegistryAccessError("Error access to : {url} \nCode Error : {status_code}".format(
                url=url, status_code=resp.status_code))
        else:
            return resp.json()



# -*- coding: utf-8 -*-
import requests

from paclair.ancestries.generic import GenericAncestry, Layer
from paclair.plugins.abstract_plugin import AbstractPlugin
from paclair.exceptions import ResourceNotFoundException


class HttpPlugin(AbstractPlugin):
    """
    Http plugin
    """

    def __init__(self, clair, clair_format, base_url, verify):
        """
        Constructor

        :param clair: ClairRequest object
        :param clair_format: Clair format
        :param base_url: base url
        :param verify: request verify certificate
        """
        super().__init__(clair, clair_format)
        self.base_url = base_url
        self.verify = verify

    @staticmethod
    def _clean_name(name):
        """
        Delete extension and path

        :param name: the name to clean
        :return:
        """
        # Delete ext
        if name.endswith('.tar.gz'):
            name = name[:-7]
        elif name.endswith('.tgz'):
            name = name[:-4]

        # Delete subpath
        _, _, name = name.rpartition('/')
        return name

    def create_ancestry(self, name):
        path = "{}/{}".format(self.base_url, name)
        result = requests.head(path, verify=self.verify)
        if result.status_code != requests.codes.ok:
            raise ResourceNotFoundException("{} not found".format(name))

        name = self._clean_name(name)
        return GenericAncestry(self._clean_name(name), self.clair_format, [Layer(name, name, path)])

    def analyse(self, name, output=None):
        return super().analyse(self._clean_name(name), output)



# -*- coding: utf-8 -*-

"""
Paclair exceptions
"""


class PaclairException(Exception):
    """
    Error base class
    """
    pass


class ConfigurationError(PaclairException):
    """
    Error reading configuration file
    """
    pass


class ClairConnectionError(PaclairException):
    """
    Error reaching Clair
    """
    def __init__(self, response):
        """
        Constructor

        :param response: requests.response
        """
        super().__init__(response.reason)
        self.response = response


class ResourceNotFoundException(PaclairException):
    """
    Resource not found
    """
    pass


class PluginNotFoundException(PaclairException):
    """
    Unknown plugin
    """
    pass


class RegistryAccessError(PaclairException):
    """
    Error reaching registry
    """
    pass



import logging
import unittest
from unittest.mock import patch

import requests_mock
from elasticsearch import Elasticsearch

from paclair.api.clair_requests_v1 import ClairRequestsV1
from paclair.exceptions import ResourceNotFoundException, ClairConnectionError
from paclair.plugins.es_plugin import EsPlugin


class TestEsPlugin(unittest.TestCase):
    """
    Test de l'objet EsPlugin
    """
    
    def setUp(self):
        self.es = EsPlugin(ClairRequestsV1('http://localhost:6060'), [{'host': '172.18.8.10', 'port': 9200}],
                           'hbx-confo_factssysinfo', 'factssysinfo')
        self.layer_name = "toto"

    def test_push_not_found(self):
        """
        Test de la méthode push
        """
        with patch.object(Elasticsearch, "search", return_value={'hits': {'total': 0, 'hits': []}}),\
                self.assertRaises(ResourceNotFoundException):
            self.es.push(self.layer_name)

    @requests_mock.mock()
    def test_push_not_in_db(self, m):
        """
        Push avec layer non présent dans CLair
        """
        search = {'hits': {'total': 1, 'hits': [{'_id': "tutu"}]}}
        m.delete(self.es.clair.url + self.es.clair._CLAIR_DELETE_URI.format(self.layer_name),
                 status_code=404, reason="Not Found")
        m.post(self.es.clair.url + self.es.clair._CLAIR_POST_URI)
        with patch.object(Elasticsearch, "search", return_value=search):
            self.es.push(self.layer_name)

        path = 'http://172.18.8.10:9200/hbx-confo_factssysinfo/factssysinfo/tutu/_source?_source_include=sysinfo_clair'
        clair_data = {'Layer': {'Path': path, 'Format': 'Legacy', 'Name': 'toto'}}
        self.assertEqual(m.last_request.json(), clair_data)

    @requests_mock.mock()
    def test_push_already_in_db(self, m):
        """
        Push et layer présent dans Clair
        """
        search = {'hits': {'total': 1, 'hits': [{'_id': "tutu"}]}}
        m.delete(self.es.clair.url + self.es.clair._CLAIR_DELETE_URI.format(self.layer_name))
        m.post(self.es.clair.url + self.es.clair._CLAIR_POST_URI)
        with patch.object(Elasticsearch, "search", return_value=search):
            self.es.push(self.layer_name)

    @requests_mock.mock()
    def test_push_clair_error(self, m):
        """
        Push et erreur Clair
        """
        search = {'hits': {'total': 1, 'hits': [{'_id': "tutu"}]}}
        m.delete(self.es.clair.url + self.es.clair._CLAIR_DELETE_URI.format(self.layer_name))
        m.post(self.es.clair.url + self.es.clair._CLAIR_POST_URI, status_code=502)
        with patch.object(Elasticsearch, "search", return_value=search), self.assertRaises(ClairConnectionError):
            self.es.push(self.layer_name)


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



import json
import logging
import os
import unittest
from unittest.mock import Mock

import requests_mock

from paclair.ancestries.generic import GenericAncestry, Layer
from paclair.api.clair_requests_v3 import ClairRequestsV3
from paclair.exceptions import ResourceNotFoundException, PaclairException


class TestClairRequestV3(unittest.TestCase):
    """
    Test ClairRequest
    """

    clairURI = 'http://clair'

    def setUp(self):
        self.clair = ClairRequestsV3(self.clairURI)
        self.fixtures_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), "fixtures/clair")

    @requests_mock.mock()
    def test_get_ancestry_json(self, m):
        """
        Test get_ancestry_json
        """
        # 404
        m.get(self.clairURI + self.clair._CLAIR_ANALYZE_URI.format("ubuntu"), status_code=404, reason="Not Found")
        with self.assertRaises(ResourceNotFoundException):
            self.clair.get_ancestry_json("ubuntu")

        with open(os.path.join(self.fixtures_dir, "ubuntu_v3.json")) as f:
            m.get(self.clairURI + self.clair._CLAIR_ANALYZE_URI.format("ubuntu"), status_code=200, json=json.load(f))
        ancestry_json = self.clair.get_ancestry_json("ubuntu")
        self.assertEqual(len(ancestry_json['ancestry']['features']), 60)

    @requests_mock.mock()
    def test_get_ancestry_statistics(self, m):
        """
        Test get_ancestry_statistics
        """
        # 404
        m.get(self.clairURI + self.clair._CLAIR_ANALYZE_URI.format("ubuntu"), status_code=404, reason="Not Found")
        with self.assertRaises(ResourceNotFoundException):
            self.clair.get_ancestry_statistics("ubuntu")

        with open(os.path.join(self.fixtures_dir, "ubuntu_v3.json")) as f:
            m.get(self.clairURI + self.clair._CLAIR_ANALYZE_URI.format("ubuntu"), status_code=200, json=json.load(f))
        ancestry_stats = self.clair.get_ancestry_statistics("ubuntu")
        self.assertDictEqual(ancestry_stats, {'High': 2, 'Low': 39, 'Medium': 19, 'Negligible': 16})

    @requests_mock.mock()
    def test_get_ancestry_html(self, m):
        """
        Test get_ancestry_html
        """
        # 404
        m.get(self.clairURI + self.clair._CLAIR_ANALYZE_URI.format("ubuntu"), status_code=404, reason="Not Found")
        with self.assertRaises(ResourceNotFoundException):
            self.clair.get_ancestry_html("ubuntu")

        with open(os.path.join(self.fixtures_dir, "ubuntu_v3.json")) as f:
            m.get(self.clairURI + self.clair._CLAIR_ANALYZE_URI.format("ubuntu"), status_code=200, json=json.load(f))
        ancestry_html = self.clair.get_ancestry_html("ubuntu")

        with open(os.path.join(self.fixtures_dir, "ubuntu_v3.html")) as f:
            expected = f.read()
            self.assertEqual(expected, ancestry_html)

    @requests_mock.mock()
    def test_post_ancestry(self, m):
        """
        Test post_ancestry
        """
        self.clair._request = Mock(autospec=True)
        ancestry = GenericAncestry("ubuntu", "Docker", [Layer("aaaaa", "hash_aaaaa", "path_aaaaa"),
                                                        Layer("bbbbb", "hash_bbbbb", "path_bbbbb")])
        expected_json = {'ancestry_name': 'ubuntu', 'format': 'Docker', 'layers':
            [{'hash': 'hash_aaaaa', 'path': 'path_aaaaa', 'headers': None},
             {'hash': 'hash_bbbbb', 'path': 'path_bbbbb', 'headers': None}]}
        self.clair.post_ancestry(ancestry)
        self.clair._request.assert_called_with('POST', '/ancestry', json=expected_json)

    def test_delete_ancestry(self):
        """
        Test delete_ancestry
        """
        ancestry = GenericAncestry("ubuntu", "Docker", [Layer("aaaaa", "hash_aaaaa", "path_aaaaa"),
                                                        Layer("bbbbb", "hash_bbbbb", "path_bbbbb")])
        with self.assertRaises(PaclairException):
            self.clair.delete_ancestry(ancestry)


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



# -*- coding: utf-8 -*-
from paclair.ancestries.generic import GenericAncestry, Layer


class DockerAncestry(GenericAncestry):
    """
    Construct an ancestry from a docker image
    """

    def __init__(self, docker_image):
        """
        Constructor

        :param docker_image:  docker image
        """
        super().__init__("", "Docker")

        # headers
        headers = {'Authorization': "Bearer {}".format(docker_image.token)}
        partial_path = docker_image.registry.get_blobs_url(docker_image, '{digest}')

        # Create layers
        parent = ""
        for layer in docker_image.get_layers():
            self.layers.append(Layer("{}_{}".format(layer, docker_image.short_sha), layer,
                                     partial_path.format(digest=layer), headers, parent))
            parent = layer
        self.name = self.layers[-1].name



# -*- coding: utf-8 -*-

from paclair.api.abstract_clair_requests import AbstractClairRequests
from paclair.exceptions import PaclairException
from paclair.struct import InsensitiveCaseDict


class ClairRequestsV3(AbstractClairRequests):
    """
    Request Clair helper
    """

    _CLAIR_ANALYZE_URI = "/ancestry/{}?with_vulnerabilities=1&with_features=1"
    _CLAIR_POST_URI = "/ancestry"
    #_CLAIR_DELETE_URI = "/v1/ancestry/{}"

    def post_ancestry(self, ancestry):
        """
        Post ancestry to Clair

        :param ancestry: ancestry to push
        """
        json = ancestry.to_json()
        json['ancestry_name'] = ancestry.name.replace(':', '_')
        return self._request('POST', self._CLAIR_POST_URI, json=json)

    def get_ancestry_json(self, ancestry):
        """
        Analyse an ancestry

        :param ancestry: ancestry (name) to analyse
        :return: json
        """
        response = self._request('GET', self._CLAIR_ANALYZE_URI.format(ancestry.replace(':', '_')))
        return response.json()

    def delete_ancestry(self, ancestry):
        """
        Delete ancestry from Clair

        :param ancestry: ancestry to delete
        """
        raise PaclairException("Delete is not available for V3 api")

    def _iter_features(self, clair_json):
        for feature in clair_json.get("ancestry", {}).get("features", {}):
            yield InsensitiveCaseDict(feature)



import json
import os
import unittest

import requests_mock

from paclair import DOCKER_HUB_TOKEN_REQUEST, DOCKER_HUB_DOMAIN
from paclair.docker.docker_image import DockerImage
from paclair.docker.docker_registry import DockerRegistry
from paclair.exceptions import RegistryAccessError

###################################################
# Global Variables
###################################################

FIXTURES_FOLDER = os.path.join(os.path.dirname(os.path.realpath(__file__)), "fixtures/")


class TestDockerImage(unittest.TestCase):
    """
    Test de l'objet DockerImage
    """
    EXPECTED_LAYERS = ['sha256:cf4e6d5921e259d673f63d820189f85cbfe8d0395f2c622a1a319fcfe33ed300',
                       'sha256:a3ed95caeb02ffe68cdd9fd84406680ae93d633cb16422d00e8a7c22955b46d4',
                       'sha256:08d02af113c958358618db8f237ce540588a62c590bfd1bbad7eba3eea97a02d',
                       'sha256:b36fdace5c59dfc1c02752efd7d6549b4dcb7ca5eb29e1c2b5f79a7a931aa248',
                       'sha256:64d3843af9ec26cad084b1e517243c3e2a4f309a8471d0e7a282b5c8b6ebbdce',
                       'sha256:ef25dcef04e03fa71fa4837dc77fe0d35f5dcc62a282b1852370848aeb140dce',
                       'sha256:1359e66c6258c932de98053676348b8923854079d85697b192684fd9d10a58be',
                       'sha256:cdd7b9a0b9a3da1d891f54aab8559a324a210495f2324abbb6ab78ae387ae9db',
                       'sha256:1819a55d3931f29312b52d14f1d4e8fb9be94a870f24261168464d0ff5159507',
                       'sha256:f01836d9fae29e08f1ec1377039152f46fef36627201e2828cb33a60fb33ebac',
                       'sha256:5e530ab118e05274008baf34b470a56a2273b0349653400410c6a66e98dc0061']

    EXPECTED_LAYERS_V2 = ['sha256:e692418e4cbaf90ca69d05a66403747baa33ee08806650b51fab815ad7fc331f',
                          'sha256:3c3a4604a545cdc127456d94e421cd355bca5b528f4a9c1905b15da2eb4a4c6b',
                          'sha256:ec4b8955958665577945c89419d1af06b5f7636b4ac3da7f12184802ad867736']

    def setUp(self):
        with open(FIXTURES_FOLDER + 'docker/manifest.json') as data_file:
            self.manifest_json = json.load(data_file)
        with open(FIXTURES_FOLDER + 'docker/manifestv2.json') as data_file:
            self.manifestv2_json = json.load(data_file)
        with open(FIXTURES_FOLDER + '/docker/token.json') as data_file:
            self.token_json = json.load(data_file)
        self.token = self.token_json['token']

    @requests_mock.mock()
    def test_get_layers_docker_hub(self, m):
        """
        Test de la méthode get_layer
        """
        registry = DockerRegistry(DOCKER_HUB_DOMAIN, token_url=DOCKER_HUB_TOKEN_REQUEST)
        image = DockerImage('library/monimage', registry, tag='tag')
        m.get(DOCKER_HUB_TOKEN_REQUEST.format(image=image), json=self.token_json)
        m.get(registry.get_manifest_url(image), json=self.manifest_json,
              headers={'Authorization': 'Bearer {token}'.format(token=self.token)})

        layers = image.get_layers()
        self.assertEqual(layers, self.EXPECTED_LAYERS)

    @requests_mock.mock()
    def test_sha(self, m):
        """
        Test sha method
        """
        registry = DockerRegistry(DOCKER_HUB_DOMAIN, token_url=DOCKER_HUB_TOKEN_REQUEST)
        image = DockerImage('library/monimage', registry, tag='tag')
        m.get(DOCKER_HUB_TOKEN_REQUEST.format(image=image), json=self.token_json)
        m.get(registry.get_manifest_url(image), json=self.manifest_json,
              headers={'Authorization': 'Bearer {token}'.format(token=self.token)})
        self.assertEqual("58c0a48e3b6a2c9f3b1d8166e096789f20b769f538f4aecaa73d5eae15672b2b", image.sha)
        self.assertEqual("58c0a48e3b6a", image.short_sha)

    @requests_mock.mock()
    def test_get_layers_docker_hub_v2(self, m):
        """
        Test de la méthode get_layer
        """
        registry = DockerRegistry(DOCKER_HUB_DOMAIN, token_url=DOCKER_HUB_TOKEN_REQUEST)
        image = DockerImage('library/monimage', registry, tag='tag')
        m.get(DOCKER_HUB_TOKEN_REQUEST.format(image=image), json=self.token_json)
        m.get(registry.get_manifest_url(image), json=self.manifestv2_json,
              headers={'Authorization': 'Bearer {token}'.format(token=self.token)})

        layers = image.get_layers()
        self.assertEqual(layers, self.EXPECTED_LAYERS_V2)

    @requests_mock.mock()
    def test_get_layers_artifactory(self, m):
        """
        Test de la méthode get_layer
        """
        artifactory_registry = 'registry.artifactory.net'
        token_url = "https://{registry.domain}/api/docker/{image.repository}/v2/token?service={registry.domain}"
        registry = DockerRegistry(artifactory_registry, token_url=token_url, protocol="http")
        image = DockerImage('monimage', registry)
        m.get(token_url.format(image=image, registry=registry), json=self.token_json)
        m.get(registry.get_manifest_url(image), json=self.manifest_json,
              headers={'Authorization': 'Bearer {token}'.format(token=self.token)})

        layers = image.get_layers()
        self.assertEqual(layers, self.EXPECTED_LAYERS)

    @requests_mock.mock()
    def test_get_layers_artifactory_v2(self, m):
        """
        Test de la méthode get_layer
        """
        artifactory_registry = 'registry.artifactory.net'
        token_url = "https://{registry.domain}/api/docker/{image.repository}/v2/token?service={registry.domain}"
        registry = DockerRegistry(artifactory_registry, token_url=token_url, protocol="http")
        image = DockerImage('monimage', registry)
        m.get(token_url.format(image=image, registry=registry), json=self.token_json)
        m.get(registry.get_manifest_url(image), json=self.manifestv2_json,
              headers={'Authorization': 'Bearer {token}'.format(token=self.token)})

        layers = image.get_layers()
        self.assertEqual(layers, self.EXPECTED_LAYERS_V2)

    @requests_mock.mock()
    def test_get_layers_other_registery(self, m):
        """
        Test de la méthode get_layer
        """
        domain_lambda = 'registery.host'
        registry = DockerRegistry(domain_lambda, token_url="https://token_url")
        image = DockerImage('monimage', registry)
        # Token
        m.get("https://token_url", json=self.token_json)
        # Manifest
        m.get(registry.get_manifest_url(image), json=self.manifest_json,
              headers={'Authorization': 'Bearer {token}'.format(token=self.token)})

        layers = image.get_layers()
        self.assertEqual(layers, self.EXPECTED_LAYERS)

    @requests_mock.mock()
    def test_get_layers_other_registery_v2(self, m):
        """
        Test de la méthode get_layer
        """
        domain_lambda = 'registery.host'
        registry = DockerRegistry(domain_lambda, token_url="https://token_url")
        image = DockerImage('monimage', registry)
        # Token
        m.get("https://token_url", json=self.token_json)
        # Manifest
        m.get(registry.get_manifest_url(image), json=self.manifestv2_json,
              headers={'Authorization': 'Bearer {token}'.format(token=self.token)})

        layers = image.get_layers()
        self.assertEqual(layers, self.EXPECTED_LAYERS_V2)

    @requests_mock.mock()
    def test_manifest_http_error(self, m):
        """
        Http Error en cas d'erreur d'accès à la registry

        """
        domain_lambda = 'registery.host'
        registry = DockerRegistry(domain_lambda, token_url="https://token_url")
        image = DockerImage('monimage', registry)
        # Token
        m.get("https://token_url", json=self.token_json)
        # Manifest
        m.register_uri('GET', registry.get_manifest_url(image), json=self.manifest_json,
                       headers={'Authorization': 'Beare {token}'.format(token=self.token)}, status_code=401)

        with self.assertRaises(RegistryAccessError):
            image.get_layers()

    @requests_mock.mock()
    def test_token_endpoint(self, m):
        """
        Test de la méthode token_endpoint
        """
        m.register_uri('GET', "https://{}/v2/".format(DOCKER_HUB_DOMAIN), status_code=401,
                       headers={'Www-Authenticate':
                                    'Bearer realm="https://auth.docker.io/token",service="registry.docker.io"'})
        registry = DockerRegistry(DOCKER_HUB_DOMAIN)
        self.assertEqual(registry.token_url, DOCKER_HUB_TOKEN_REQUEST)

        m.register_uri('GET', "https://{}/v2/".format(DOCKER_HUB_DOMAIN), headers={})
        with self.assertRaises(RegistryAccessError):
            registry = DockerRegistry(DOCKER_HUB_DOMAIN)
            print(registry.token_url)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import json
from abc import ABCMeta, abstractmethod
from bottle import template
from pkg_resources import resource_filename

import requests

from paclair.exceptions import ResourceNotFoundException, ClairConnectionError
from paclair.logged_object import LoggedObject
from paclair.struct import InsensitiveCaseDict


class AbstractClairRequests(LoggedObject):
    """
    Base Api
    """
    __metaclass__ = ABCMeta

    def __init__(self, clair_url, verify=True, html_template=None):
        """
        Constructor

        :param clair_url: Clair api URL
        :param verify: request verify certificate
        :param html_template: html template
        """
        super().__init__()
        self.url = clair_url
        self.verify = verify
        self.html_template = html_template or resource_filename(__name__, 'template/report.tpl')

    def _request(self, method, uri, **kwargs):
        """
        Execute http method on uri

        :param method: http verb
        :param uri: uri to request
        :param kwargs: other params
        :return : server's response
        """
        url = self.url + uri
        self.logger.debug("Requesting {} on {}".format(method, url))
        response = requests.request(method, url, verify=self.verify, **kwargs)
        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError:
            self.logger.error("Bad http code {} requesting Clair".format(response.status_code))
            if response.reason == "Not Found":
                raise ResourceNotFoundException("Resource not found")
            raise ClairConnectionError(response)
        return response

    def get_ancestry(self, ancestry, output=None):
        """
        Analyse an ancestry

        :param ancestry: ancestry (name) to analyse
        :param output: change default output (choose between stats and html)
        :return: choosed output
        """
        if output == "stats":
            return self.get_ancestry_statistics(ancestry)
        elif output == "html":
            return self.get_ancestry_html(ancestry)
        else:
            return self.get_ancestry_json(ancestry)

    @abstractmethod
    def get_ancestry_json(self, ancestry):
        """
        Get json output for ancestry

        :param ancestry: ancestry (name) to analyse
        :return: json
        """
        raise NotImplementedError("Implement in sub classes")

    def get_ancestry_statistics(self, ancestry):
        """
        Get statistics for ancestry

        :param ancestry: ancestry (name) to analyse
        :return: statistics (dict)
        """
        clair_json = self.get_ancestry_json(ancestry)
        result = {}
        for feature in self._iter_features(clair_json):
            for vuln in feature.get("vulnerabilities", []):
                vuln = InsensitiveCaseDict(vuln)
                if "fixedBy" in vuln:
                    result[vuln["severity"]] = result.setdefault(vuln["severity"], 0) + 1
        return result

    def get_ancestry_html(self, ancestry):
        """
        Get html output for ancestry

        :param ancestry: ancestry (name) to analyse
        :return: html
        """
        clair_info = []
        for feature in self._iter_features(self.get_ancestry_json(ancestry)):
            for vuln in feature.get("vulnerabilities", {}):
                vuln = InsensitiveCaseDict(vuln)
                # metadata is a string in v3 and dict in v1
                metadata = vuln.get("Metadata", {})
                if isinstance(metadata, str):
                    try:
                        metadata = json.loads(metadata)
                    except ValueError:
                        metadata = {}
                cvss = metadata.get('NVD', {}).get("CVSSv2", {})
                cvss_vector = self.split_vectors(cvss.get('Vectors', ""))
                clair_info.append({"ID": len(clair_info),
                                   "CVE": vuln.get("Name"),
                                   "SEVERITY": vuln.get("Severity"),
                                   "PACKAGE": feature.get("Name"),
                                   "CURRENT": feature.get("Version"),
                                   "FIXED": vuln.get("FixedBy", ""),
                                   "INTRODUCED": feature.get("AddedBy"),
                                   "DESCRIPTION": vuln.get("Description"),
                                   "LINK": vuln.get("Link"),
                                   "VECTORS": cvss_vector,
                                   "SCORE": cvss.get("Score")})
        return template(self.html_template, info=clair_info)

    @abstractmethod
    def post_ancestry(self, ancestry):
        """
        Post ancestry to Clair

        :param ancestry: ancestry to push
        """
        raise NotImplementedError("Implement in sub classes")

    @abstractmethod
    def delete_ancestry(self, ancestry):
        """
        Delete ancestry from Clair

        :param ancestry: ancestry to delete
        """
        raise NotImplementedError("Implement in sub classes")

    @abstractmethod
    def _iter_features(self, clair_json):
        """
        Iterate over features from clair_json via CaseInsensitiveDict

        :param clair_json: json to iterate overs
        """
        raise NotImplementedError("Implement in sub classes")

    @staticmethod
    def split_vectors(vectors_str):
        """
        Convert nvd metadata ("AV:N/AC:L/Au:N/C:N/I:N") into a dict

        :param vectors_str: string like "AV:N/AC:L/Au:N/C:N/I:N"
        :return: dict
        """
        npc = {'N': 'None', 'P': 'Partial', 'C': 'Complete'}
        vectors = {'AV': ('Access Vector', {'L': 'Local', 'A': 'Adjacent Network', 'N': 'Network'}),
                   'AC': ('Access Complexity', {'H': 'High', 'M': 'Medium', 'L': 'Low'}),
                   'Au': ('Authentication', {'S': 'Single', 'M': 'Multiple', 'N': 'None'}),
                   'C': ('Confidentiality impact', npc),
                   'I': ('Integrity impact', npc),
                   'A': ('Availability impact', npc)}
        try:
            dict_vectors = dict((vector.split(':') for vector in vectors_str.split('/')))
        except ValueError:
            dict_vectors = {}

        return {v[0]: v[1].get(dict_vectors.get(metric), "") for metric, v in vectors.items()}



import json
import logging
import os
import unittest
from unittest.mock import Mock

import requests_mock

from paclair.ancestries.generic import GenericAncestry, Layer
from paclair.api.clair_requests_v1 import ClairRequestsV1
from paclair.exceptions import ResourceNotFoundException, ClairConnectionError, PaclairException


class TestClairRequestV1(unittest.TestCase):
    """
    Test ClairRequest
    """

    clairURI = 'http://clair'

    def setUp(self):
        self.clair = ClairRequestsV1(self.clairURI)
        self.fixtures_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), "fixtures/clair")

    @requests_mock.mock()
    def test_request(self, m):
        """
        Test _request method
        """
        # 404
        m.get(self.clairURI + "/tutu", status_code = 404, reason="Not Found")
        with self.assertRaises(ResourceNotFoundException):
            self.clair._request("GET", "/tutu")

        # Other error
        m.get(self.clairURI + "/tutu", status_code=502)
        with self.assertRaises(ClairConnectionError):
            self.clair._request("GET", "/tutu")

        # Ok
        m.get(self.clairURI + "/tutu", status_code=200)
        response = self.clair._request("GET", "/tutu")
        self.assertEqual(response.status_code, 200)

    @requests_mock.mock()
    def test_get_ancestry_json(self, m):
        """
        Test get_ancestry_json
        """
        # 404
        m.get(self.clairURI + self.clair._CLAIR_ANALYZE_URI.format("ubuntu"), status_code=404, reason="Not Found")
        with self.assertRaises(ResourceNotFoundException):
            self.clair.get_ancestry_json("ubuntu")

        with open(os.path.join(self.fixtures_dir, "ubuntu_v1.json")) as f:
            m.get(self.clairURI + self.clair._CLAIR_ANALYZE_URI.format("ubuntu"), status_code=200, json=json.load(f))
        ancestry_json = self.clair.get_ancestry_json("ubuntu")
        self.assertEqual(len(ancestry_json["Layer"]["Features"]), 148)

    @requests_mock.mock()
    def test_get_ancestry_statistics(self, m):
        """
        Test get_ancestry_statistics
        """
        # 404
        m.get(self.clairURI + self.clair._CLAIR_ANALYZE_URI.format("ubuntu"), status_code=404, reason="Not Found")
        with self.assertRaises(ResourceNotFoundException):
            self.clair.get_ancestry_statistics("ubuntu")

        with open(os.path.join(self.fixtures_dir, "ubuntu_v1.json")) as f:
            m.get(self.clairURI + self.clair._CLAIR_ANALYZE_URI.format("ubuntu"), status_code=200, json=json.load(f))
        ancestry_stats = self.clair.get_ancestry_statistics("ubuntu")
        self.assertDictEqual(ancestry_stats, {'Medium': 1, 'High': 1, 'Low': 1})

    @requests_mock.mock()
    def test_get_ancestry_html(self, m):
        """
        Test get_ancestry_html
        """
        # 404
        m.get(self.clairURI + self.clair._CLAIR_ANALYZE_URI.format("ubuntu"), status_code=404, reason="Not Found")
        with self.assertRaises(ResourceNotFoundException):
            self.clair.get_ancestry_html("ubuntu")

        with open(os.path.join(self.fixtures_dir, "ubuntu_v1.json")) as f:
            m.get(self.clairURI + self.clair._CLAIR_ANALYZE_URI.format("ubuntu"), status_code=200, json=json.load(f))
        ancestry_html = self.clair.get_ancestry_html("ubuntu")
        with open(os.path.join(self.fixtures_dir, "ubuntu_v1.html")) as f:
            expected = f.read()
            self.assertEqual(expected, ancestry_html)

    @requests_mock.mock()
    def test_post_ancestry(self, m):
        """
        Test post_ancestry
        """
        self.clair._request = Mock(autospec=True)
        ancestry = GenericAncestry("ubuntu", "Docker", [Layer("aaaaa", "hash_aaaaa", "path_aaaaa"),
                                                        Layer("bbbbb", "hash_bbbbb", "path_bbbbb")])
        self.clair.post_ancestry(ancestry)
        self.clair._request.assert_any_call('POST', self.clair._CLAIR_POST_URI,
                                               json={'Layer': {'Name': 'aaaaa', 'Path': 'path_aaaaa', 'Format': 'Docker'}})
        self.clair._request.assert_any_call('POST', self.clair._CLAIR_POST_URI,
                                            json={'Layer': {'Name': 'bbbbb', 'Path': 'path_bbbbb', 'Format': 'Docker',
                                                            'ParentName': 'aaaaa'}})
    def test_delete_ancestry(self):
        """
        Test delete_ancestry
        """
        self.clair._request = Mock(autospec=True)
        ancestry = GenericAncestry("ubuntu", "Docker", [Layer("aaaaa", "hash_aaaaa", "path_aaaaa"),
                                                        Layer("bbbbb", "hash_bbbbb", "path_bbbbb")])
        self.clair.delete_ancestry(ancestry)
        self.clair._request.assert_any_call("DELETE", self.clair._CLAIR_DELETE_URI.format("aaaaa"))
        self.clair._request.assert_any_call("DELETE", self.clair._CLAIR_DELETE_URI.format("bbbbb"))


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



# -*- coding: utf-8 -*-
import json

from paclair.exceptions import PluginNotFoundException
from paclair.logged_object import LoggedObject
from paclair.config_reader import ConfigReader
from paclair.exceptions import PaclairException, ConfigurationError
from yaml import YAMLError
import logging
import logging.handlers
import argparse
import os
import sys

DEFAULT_CONFIG_FILE = "/etc/paclair.conf"


class PaClair(LoggedObject):
    """
    Class Main
    """
    def __init__(self, config_file=None):
        """
        Constructor

        :param config_file: configuration file
        """
        super().__init__()
        self._config_reader = ConfigReader(config_file or DEFAULT_CONFIG_FILE)
        try:
            self._plugins = self._config_reader.read_plugins('Plugins')
        except YAMLError:
            raise ConfigurationError("Incorrect configuration file")

    def _check_plugin(self, plugin):
        """
        Check if plugin is available

        :param plugin: plugin to check
        :raises PluginNotFoundException: if not found
        """
        if plugin not in self._plugins:
            raise PluginNotFoundException("Plugin {} is unknown".format(plugin))

    def analyse(self, plugin, name, delete=False, output=None):
        """
        Analyse a layer

        :param plugin: plugin's name
        :param name: resource to analyse
        :param delete: delete after analyse
        :param output: change default output
        :return: string
        :raises ResourceNotFoundException: if layer not found
        :raise ClairConnectionError: if an error occurs requesting Clair
        """
        self._check_plugin(plugin)

        self.logger.debug("Analysing {}".format(name))
        result = self._plugins[plugin].analyse(name, output)

        if output == "stats":
            result = '\n'.join(("{}: {}".format(k, v) for k, v in result.items()))
        elif output != "html":
            result = json.dumps(result)

        if delete:
            self.logger.debug("Deleting  {}".format(name))
            self._plugins[plugin].delete(name)
        return result

    def push(self, plugin, name):
        """
        Push layer to Clair

        :param plugin: plugin's name
        :param name: resource to push
        :return:
        """
        self._check_plugin(plugin)

        self.logger.debug("Push {} with plugin {}".format(name, plugin))
        self._plugins[plugin].push(name)

    def delete(self, plugin, name):
        """
        Delete image from Clair

        :param plugin: plugin's name
        :param name: resource to delete
        :raises ResourceNotFoundException: if layer not found
        :raise ClairConnectionError: if an error occurs requesting Clair
        """
        self._check_plugin(plugin)

        self.logger.debug("Delete {} with plugin {}".format(name, plugin))
        self._plugins[plugin].delete(name)


def main():
    """
    Main

    """
    # Create parser
    parser = argparse.ArgumentParser()
    parser.add_argument("--debug", help="Debug mode", action="store_true")
    parser.add_argument("--syslog", help="Log to syslog", action="store_true")
    parser.add_argument("--conf", help="Conf file", action="store", default=DEFAULT_CONFIG_FILE)
    parser.add_argument("plugin", help="Plugin to launch", action="store")
    parser.add_argument("hosts", help="Image/hostname to analyse", nargs='+', action="store")

    # Subparsers
    subparsers = parser.add_subparsers(help="Command to launch", dest="subparser_name")
    subparsers.add_parser("push", help="Push images/hosts to Clair")
    subparsers.add_parser("delete", help="Delete images/hosts from Clair")
    parser_analyse = subparsers.add_parser("analyse", help="Analyse images/hosts already pushed to Clair")
    parser_analyse.add_argument("--output-format", help="Change default output format (default: json)",
                                choices=['stats', 'html'])
    parser_analyse.add_argument("--output-report", help="Change report location (default: logger)",
                                choices=['file', 'term'])
    parser_analyse.add_argument("--output-dir", help="Change output directory (default: current)", action="store",
                                default=".")
    parser_analyse.add_argument("--delete", help="Delete after analyse", action="store_true")

    # Parse args
    args = parser.parse_args()

    # Init logger
    logger = logging.getLogger()
    if args.debug:
        logger.setLevel(level=logging.DEBUG)
    else:
        logger.setLevel(level=logging.INFO)

    if args.syslog:
        # Logger format
        formatter = logging.Formatter(
            'PACLAIR[{}]: ({}) %(levelname).1s %(message)s'.format(os.getpid(), os.getenv('USER')))
        # Syslog Handler
        syslog_handler = logging.handlers.SysLogHandler(address='/dev/log')
        syslog_handler.setFormatter(formatter)
        logger.addHandler(syslog_handler)
    else:
        logger.addHandler(logging.StreamHandler())

    # Run
    try:
        paclair_object = PaClair(args.conf)
    except (OSError, ConfigurationError) as error:
        logger.error(error)
        sys.exit(1)

    for host in args.hosts:
        try:
            if args.subparser_name == "push":
                paclair_object.push(args.plugin, host)
                logger.info("Pushed {} to Clair.".format(host))
            elif args.subparser_name == "delete":
                paclair_object.delete(args.plugin, host)
                logger.info("{} was deleted from Clair.".format(host))
            elif args.subparser_name == "analyse":
                result = paclair_object.analyse(args.plugin, host, args.delete, args.output_format)
                # Report
                if args.output_report == "term":
                    print(result)
                elif args.output_report == "file":
                    filename = os.path.join(args.output_dir, '{}.{}'.format(host.replace('/', '_'), args.output_format
                                                                            or 'json'))
                    try:
                        with open(filename, "w", encoding="utf-8") as report_file:
                            report_file.write(result)
                    except (OSError, IOError):
                        logger.error("Can't write in directory: {}".format(args.output_dir))
                        sys.exit(4)
                else:
                    logger.info(result)
            else:
                parser.print_help()
        except PluginNotFoundException as error:
            logger.error("Can't find plugin {} in configuration file.".format(args.plugin))
            logger.error(error)
            sys.exit(2)
        except PaclairException as error:
            logger.error("Error treating {}".format(host))
            logger.error(error)
            sys.exit(3)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

import logging


class LoggedObject:
    """
    Easy access to logging
    """

    def __init__(self, logger=None):
        """
        Constructor

        :param logger: logger
        """
        self.logger = logger or logging.getLogger(__name__)



#-*- coding: UTF-8 -*-

from setuptools import setup
from codecs import open

with open('README.rst', 'r', 'utf-8') as f:
    readme = f.read()

setup(name="paclair",
      version="3.0.0",
      description="Push and Analyse containers with Clair",
      long_description=readme,
      author="Grégoire UNBEKANDT",
      url="https://github.com/yebinama/paclair",
      packages=["paclair", "paclair/api", "paclair/plugins", "paclair/docker", "paclair/ancestries"],
      package_data={
        "paclair": ['api/template/report.tpl']
      },
      install_requires=[
          'elasticsearch',
          'requests>=2.4.2',
          'pyyaml',
          'bottle'
      ],
      command_options={
               'build_sphinx': {
                   'project': ('setup.py', "paclair"),
                   'version': ('setup.py', "3.0.0"),
                   'release': ('setup.py', "3.0.0"),
                   'build_dir': ('setup.py', 'doc/sphinx/_build/')}},
     entry_points={
         'console_scripts': [
             'paclair = paclair.__main__:main'
         ]
     },
     classifiers=[
         "Programming Language :: Python",
         "Programming Language :: Python :: 3",
         "Programming Language :: Python :: 3.2",
         "Programming Language :: Python :: 3.3",
         "Programming Language :: Python :: 3.4",
         "Programming Language :: Python :: 3.5",
         "Programming Language :: Python :: 3.6",
     ],
)



# -*- coding: utf-8 -*-

"""
Plugin de base
"""

from abc import ABCMeta, abstractmethod
from paclair.logged_object import LoggedObject


class AbstractPlugin(LoggedObject):
    """
    Base Plugin
    """
    __metaclass__ = ABCMeta


    def __init__(self, clair, clair_format):
        """
        Constructor

        :param clair_format: Clair format
        :param clair: ClairRequests object
        """
        super().__init__()
        self.clair_format = clair_format
        self.clair = clair

    def analyse(self, name, output=None):
        """
        Analyse a resource

        :param name: resource to analyse
        :param output: change default output (choose between stats and html)
        :return: json from clair
        """
        return self.clair.get_ancestry(name, output)

    def delete(self, name):
        """
        Delete a resource

        :param name: resource's name
        """
        return self.clair.delete_ancestry(self.create_ancestry(name))

    def push(self, name):
        """
        Push to Clair

        :param name: resource's name
        """
        return self.clair.post_ancestry(self.create_ancestry(name))

    @abstractmethod
    def create_ancestry(self, name):
        """
        Create ancestry associated with this plugin

        :param name: name
        :return: Ancestry object
        """
        raise NotImplementedError("Implement in sub classes")
